using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace FallingBlocks
{
    class Playfield
    {
        //Playfield dimensions 
        private const int mWidth = 14;
        private const int mHeight = 26;
        private int[,] mPlayfield = new int[mWidth, mHeight];

        private int mBlockSize = 28;
        private int mNumberOfBlocks = 4;
        
        public Playfield()
        {
        }

        //Description: Render the playfield
        public void Draw(SpriteBatch theSpriteBatch, Texture2D theBlockTexture, Color theColor)
        {
            int aTextureStartX = 5 * (int)mBlockSize;
            int aStartX = 350;
            int aStartY = 10;
            for (int aXPosition = 0; aXPosition < mWidth; aXPosition++)
            {
                for (int aYPosition = 0; aYPosition < mHeight; aYPosition++)
                {
                    if ((mPlayfield[aXPosition, aYPosition] >= 0))
                    {
                        aTextureStartX = mPlayfield[aXPosition, aYPosition] * 28;
                        int aPositionX = (int)(aStartX + (aXPosition * mBlockSize));
                        int aPositionY = (int)(aStartY + (aYPosition * mBlockSize));

                        theSpriteBatch.Draw(theBlockTexture, new Rectangle(aPositionX, aPositionY, mBlockSize, mBlockSize), new Microsoft.Xna.Framework.Rectangle(aTextureStartX, 0, 27, 27), theColor);

                    }
                }
            }
        }

        //Description: Check to see if the current shape in play has hit something
        public bool CheckCollision(Shape theShape)
        {
            for (int aBlock = 0; aBlock < mNumberOfBlocks; aBlock++)
            {
                if (theShape.PositionX < 0)
                {
                    return true;
                }

                //Check to see if one of the blocks that make up a shape is moving outside of the width or height of
                //the established playing area
                if (((theShape.PositionX + theShape.CurrentShape[theShape.Rotation, aBlock, 0] > mWidth - 1) | (theShape.PositionY + theShape.CurrentShape[theShape.Rotation, aBlock, 1] > mHeight - 1)))
                {
                    return true;
                }

                //If the current shapes position is outside of the top of the playing area, then
                //indicate that the game is over
                if (theShape.PositionY < 0)
                {
                    //base.myStage = eStage.GameOver;
                    return true;
                }
                if ((mPlayfield[theShape.PositionX + theShape.CurrentShape[theShape.Rotation, aBlock, 0], theShape.PositionY + theShape.CurrentShape[theShape.Rotation, aBlock, 1]] >= 0))
                {
                    return true;
                }
            }
            return false;
        }

        //Description: Check for full rows and remove them 
        public int CheckRows()
        {
            int aNumberOfLinesCleared = 0;

            for (int aYPosition = mHeight - 1; aYPosition > 0; aYPosition--)
            {
                //Check to see if the row is full at the current YPosition
                bool aFullRow = true;
                for (int aXPosition = 0; aXPosition < mWidth; aXPosition++)
                {
                    //If an XPosition is not set, then the row is not full
                    if (mPlayfield[aXPosition, aYPosition] == -1)
                    {
                        aFullRow = false;
                        break;
                    }
                }

                //If the row is full, we want to remove the row
                if (aFullRow == true)
                {
                    for (int aY = aYPosition; aY > 1; aY--)
                    {
                        for (int aX = 0; aX < mWidth; aX++)
                        {
                            //Move all the rows up
                            mPlayfield[aX, aY] = mPlayfield[aX, aY - 1];
                        }
                    }

                    //Reset the top row
                    for (int aX = 0; aX < mWidth; aX++)
                    {
                        mPlayfield[aX, 0] = -1;
                    }


                    aNumberOfLinesCleared += 1;
                }
            }

            return aNumberOfLinesCleared;
        }

        //Description: Add a shape to the playfield
        public void AddShape(Shape theShape)
        {
            if (theShape.PositionY > mHeight - 1)
            {
                theShape.PositionY = mHeight;
            }

            //Construct a piece in the playing area and color it the current color
            for (int aBlock = 0; aBlock < mNumberOfBlocks; aBlock++)
            {
                mPlayfield[theShape.PositionX + theShape.CurrentShape[theShape.Rotation, aBlock, 0], theShape.PositionY + theShape.CurrentShape[theShape.Rotation, aBlock, 1] - 1] = theShape.Color;
            }
        }

        //Description: Clear the playfield 
        public void ClearPlayfield()
        {
            for (int aYPosition = 0; aYPosition < mHeight; aYPosition++)
            {
                for (int aXPosition = 0; aXPosition < mWidth; aXPosition++)
                {
                    mPlayfield[aXPosition, aYPosition] = -1;
                }
            }
        }

    }
}
