using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace FallingBlocks
{
    class Shape
    {
        private int mBlockSize = 28;
        public int BlockSize
        {
            get
            {
                return mBlockSize;
            }
        }

        private int mNumberOfBlocks = 4;

        public enum ShapeType
        {
        NotSet = -1,
        TShape = 0,
        LineShape = 1,
        JShape = 2,
        LShape = 3,
        SShape = 4,
        ZShape = 5,
        SquareShape = 6,
        }

        public int[, ,] TShape = new int[4, 4, 2] 
                {
                {{1, 0}, {0, 1}, {1, 1}, {2, 1}}, 
                {{1, 0}, {0, 1}, {1, 1}, {1, 2}}, 
                {{0, 0}, {1, 0}, {2, 0}, {1, 1}}, 
                {{0, 0}, {0, 1}, {1, 1}, {0, 2}}};
   
        public int[ ,,] LineShape = new int[4, 4, 2]
                {
                {{0, 0}, {1, 0}, {2, 0}, {3, 0}}, 
                {{0, 0}, {0, 1}, {0, 2}, {0, 3}}, 
                {{0, 0}, {1, 0}, {2, 0}, {3, 0}}, 
                {{0, 0}, {0, 1}, {0, 2}, {0, 3}}};

        public int[, ,] SquareShape = new int[4, 4, 2] 
              { 
              {{0, 0}, {1, 0}, {0, 1}, {1, 1}}, 
              {{0, 0}, {1, 0}, {0, 1}, {1, 1}}, 
              {{0, 0}, {1, 0}, {0, 1}, {1, 1}}, 
              {{0, 0}, {1, 0}, {0, 1}, {1, 1}} 
              };

        public int[, ,] ZShape = new int[4, 4, 2] 
              { 
              {{0, 0}, {1, 0}, {1, 1}, {2, 1}}, 
              {{1, 0}, {1, 1}, {0, 1}, {0, 2}}, 
              {{0, 0}, {1, 0}, {1, 1}, {2, 1}}, 
              {{1, 0}, {1, 1}, {0, 1}, {0, 2}} 
              };
        
        public int[,,] SShape = new int[4, 4, 2] 
            { 
            {{1, 0}, {2, 0}, {0, 1}, {1, 1}}, 
            {{0, 0}, {0, 1}, {1, 1}, {1, 2}}, 
            {{1, 0}, {2, 0}, {0, 1}, {1, 1}}, 
            {{0, 0}, {0, 1}, {1, 1}, {1, 2}} 
            };
        
        public int[,,] JShape = new int[4, 4, 2] 
              { 
              {{0, 0}, {0, 1}, {1, 1}, {2, 1}}, 
              {{1, 0}, {1, 1}, {1, 2}, {0, 2}}, 
              {{0, 0}, {1, 0}, {2, 0}, {2, 1}}, 
              {{0, 0}, {1, 0}, {0, 1}, {0, 2}} 
              };
        
        public int[,,] LShape = new int[4, 4, 2] 
              { 
              {{0, 1}, {1, 1}, {2, 1}, {2, 0}}, 
              {{0, 0}, {1, 0}, {1, 1}, {1, 2}}, 
              {{0, 0}, {1, 0}, {2, 0}, {0, 1}}, 
              {{0, 0}, {0, 1}, {0, 2}, {1, 2}} 
              };

        public Shape()
        {
        }

        private ShapeType mType = ShapeType.NotSet;
        public ShapeType Type
        {  
            get
            {
                return mType;
            } 
            set
            {
                mType = value;
                mCurrentShape = new int[3, 3, 1];
                switch (mType)
                {
                    case ShapeType.TShape:
                        {
                            mCurrentShape = TShape;
                            mColor = 0;
                            break;
                        }
                    case ShapeType.LineShape:
                        {
                            mCurrentShape = LineShape;
                            mColor = 1;
                            break;
                        }
                    case ShapeType.JShape:
                        {
                            mCurrentShape = JShape;
                            mColor = 2;
                            break;
                        }
                    case ShapeType.LShape:
                        {
                            mCurrentShape = LShape;
                            mColor = 3;
                            break;
                        }
                    case ShapeType.SShape:
                        {
                            mCurrentShape = SShape;
                            mColor = 4;
                            break;
                        }
                    case ShapeType.ZShape:
                        {
                            mCurrentShape = ZShape;
                            mColor = 5;
                            break;
                        }
                    case ShapeType.SquareShape:
                        {
                            mCurrentShape = SquareShape;
                            mColor = 6;
                            break;
                        }
                }
             }
        }

        private int[,,] mCurrentShape = null;
        public int[,,] CurrentShape
        {
            get
            {
                return mCurrentShape;
            }
        }

        private int mRotation = 0;
        public int Rotation
        {
            get 
            {
                return mRotation;
            }
            set 
            {
                mRotation = value;
            }
        }

        private int mColor = 0;
        public int Color
        {
            get
            {
                return mColor;
            }
        }

        private int mPositionX = 4;
        public int PositionX
        {
            get 
            {
                return mPositionX;
            }
            set 
            {
                mPositionX = value;
            }
        }

        private int mPositionY = 0;
        public int PositionY
        {
            get
            {
                return mPositionY;
            }
            set
            {
                mPositionY = value;
            }
        }

        public void Draw(int theXPosition, int theYPosition, Color theColor, SpriteBatch theSpriteBatch, Texture2D theBlockTexture)
        {
            int aTextureStartX = Color * Convert.ToInt32(mBlockSize);
            for (int aBlock = 0; aBlock < mNumberOfBlocks; aBlock++)
            {
                int aXPosition = (int)(theXPosition + (CurrentShape[Rotation, aBlock, 0] * mBlockSize));
                int aYPosition = (int)(theYPosition + (CurrentShape[Rotation, aBlock, 1] * mBlockSize));
                theSpriteBatch.Draw(theBlockTexture, new Rectangle(aXPosition, aYPosition, mBlockSize, mBlockSize), new Rectangle(aTextureStartX, 0, mBlockSize, mBlockSize), theColor);
            }
        }
        
    }
}
